# [[file:../../configuration.org::*directory configuration][directory configuration:1]]
import pathlib
import os

ARCOLOGY_BASE_DIR = pathlib.Path(os.environ.get("ARCOLOGY_BASE_DIR", "~/org")).expanduser()
ARCOLOGY_EMACS_SNIPPETS_DIR = str(pathlib.Path("~/org/cce/").expanduser())
ARROYO_BASE_DIR = str(pathlib.Path("~/arroyo-nix").expanduser())
# directory configuration:1 ends here

# [[file:../../configuration.org::*The Code][The Code:1]]
ARCOLOGY_EXTRACTORS = [
  "roam.models.Heading", # [[id:arcology/django/roam][Arcology Roam Models]]'s heading entity
  "roam.models.Reference", # [[id:arcology/django/roam][Arcology Roam Models]]'s [[id:cce/org-roam][org-roam]] reference entity
  "roam.models.Tag", # [[id:arcology/django/roam][Arcology Roam Models]]'s heading tag entity
  "roam.models.HeadingProperty", # [[id:arcology/django/roam][Arcology Roam Models]]'s heading properties map
  "roam.models.Link", # [[id:arcology/django/roam][Arcology Roam Models]] page-to-page links
  "arcology.models.Page", # [[id:arcology/django/arcology-models][The Arcology's Data Models]]'s Page entity
  "arcology.models.Feed", # [[id:arcology/django/arcology-models][The Arcology's Data Models]]'s Feed entity, probably move to its own module some day...
  "arcology.models.FeedEntry", # 
  
]

ARROYO_EXTRACTORS = {
  "emacs": "generators.models.EmacsSnippet",
  "epkgs": "generators.models.EmacsEpkg",
  "nixos": "generators.models.NixosModule",
  "home-manager": "generators.models.HomeManagerModule",
  
}
# The Code:1 ends here

# [[file:../../configuration.org::*Enumerating System Roles][Enumerating System Roles:1]]
ARROYO_ROLES = [
  "server", # [[id:20220101T195846.044283][the Wobserver Configuration with Arroyo Nixos]] $
  "endpoint", # [[id:cce/my_nixos_configuration][My NixOS configuration]]
  "settop", # [[id:20220131T152041.472624][NixOS Set Top Box]]
  "droid", # [[id:20221106T113721.266425][CCE in Nix On Droid]]
  "waterboy", # [[id:waterboy][waterboy]] icebox'd data collection platform
  
]
# Enumerating System Roles:1 ends here

# [[file:../../configuration.org::*The Code][The Code:1]]
ROAM_ALLOWED_KEYWORDS = [
  "ARCOLOGY_FEED", # Add a route that will generate an Atom feed for this page
  "ARCOLOGY_KEY", # Add a route that will generate an HTML page for this page
  "ARCOLOGY_ALLOW_CRAWL", # Whether the Arcology page will be marked as allowed in =robots.txt=
  "ARCOLOGY_TOOT_VISIBILITY", # This is set to an [[id:62538db5-d94a-47c3-9998-086ded91fd88][Mastadan]] API visibility mode like "public" or "unlisted"
  "ARROYO_EMACS_MODULE", # Instruct [[id:arroyo/django/generators][The Arroyo Generators]] to include an Emacs snippet in the init.
  "ARROYO_HOME_MODULE", # Instruct [[id:arroyo/django/generators][The Arroyo Generators]] to import the referenced home-manager module
  "ARROYO_NIXOS_MODULE", # Instruct [[id:arroyo/django/generators][The Arroyo Generators]] to import the referenced nixos module
  "ARROYO_HOME_EPKGS", # Instruct [[id:arroyo/django/generators][The Arroyo Generators]] to make a custom Emacs-Lisp package available to Arroyo Emacs
  "ARROYO_MODULE_WANTS", # Define a dependency relationship with the referenced org-roam document (this doc depends on that)
  "ARROYO_MODULE_WANTED", # Define a dependent relationship from the referenced org-roam document (that doc depends on this)
  "ARROYO_SYSTEM_ROLE", # Constrain the Nix expressions on the page to only load in this role (can be specified repeatedly)
  "ARROYO_SYSTEM_EXCLUDE", # Prevent the Nix expressions on the page from loading in the listed role (can be specified repeatedly)
  "ARROYO_DIRENV_DIR", # Load [[id:45fc2a02-fcd0-40c6-a29e-897c0ee7b1c7][direnv]] from a different directory than the loaded buffer's directory. Useful for [[id:cce/literate_programming][Org Babel]].
  "AUTHOR", # Used to populate the author in RSS feeds and page metadata
  
  "BIRTHDAY", # Used in [[id:8803fda8-20cd-4d62-878d-dcb1b7853183][People]] pages.
  "LOCATION", # Used in [[id:8803fda8-20cd-4d62-878d-dcb1b7853183][People]] pages.
  "PRONOUNS", # Used in [[id:8803fda8-20cd-4d62-878d-dcb1b7853183][People]] pages.
  "AGE_OF_MONEY", # Used in financial review of [[id:426cef6a-f77d-4b2d-8acb-0793b3cef368][Monthly Review]]
  "JPMORGAN", # Used in financial review of [[id:426cef6a-f77d-4b2d-8acb-0793b3cef368][Monthly Review]]
  "SAVINGS", # Used in financial review of [[id:426cef6a-f77d-4b2d-8acb-0793b3cef368][Monthly Review]]
  "SPENT_THIS_MONTH", # Used in financial review of [[id:426cef6a-f77d-4b2d-8acb-0793b3cef368][Monthly Review]]
  
# used for deprecated universal-aggregator feed generation
    "ARROYO_FEEDS",
    "ARROYO_FEED_CATEGORY",
    "ARROYO_FEED_URL",
# unused
    "CCE_HOME_MODULE",
    "ARROYO_EMACS_TANGLED",
    "ARROYO_MODULE_DEP",
]
# The Code:1 ends here

# [[file:../../configuration.org::*The Code][The Code:1]]
IGNORED_ROAM_TAGS = [
  "NOEXPORT", # nil
  "noexport", # nil
  "Private", # nil
  
]
# The Code:1 ends here
