# [[file:../../../interfaces.org::*Generating code using the Arroyo Generators][Generating code using the Arroyo Generators:1]]
import argparse
import pathlib
import sys

from django.core.management.base import BaseCommand, CommandError
from django.conf import settings
from django.utils.module_loading import import_string

from generators.models import DependencyRelationship

import logging
logger = logging.getLogger(__name__)

class Command(BaseCommand):
    help = 'Export Nix and other code known by the DB'
    requires_migrations_checks = True

    def add_arguments(self, parser):
        parser.add_argument('-m', '--module', type=str, help="The __class__.__name__ of the Arroyo Generator Module to collect")
        parser.add_argument('-r', '--role', default="endpoint", type=str, help="A role to restrict the modules to")
        parser.add_argument('-d', '--destination', default="-", type=str, help="Location to output generated content to. pass `-' for stdout.")
        parser.add_argument('--dry-run', default=False, action=argparse.BooleanOptionalAction, help="Only list the org-files included in the export")
# Generating code using the Arroyo Generators:1 ends here

# [[file:../../../interfaces.org::*Generating code using the Arroyo Generators][Generating code using the Arroyo Generators:2]]
    def handle(self, *args, **kwargs):
        dry_run = kwargs.get('dry_run', False)
        dest_arg = kwargs['destination']
        if dest_arg == "-":
            dest = sys.stdout
        else:
            dest = pathlib.Path(dest_arg).expanduser()
            dest = open(dest, "w")

        module = kwargs['module']
        role = kwargs['role']

        output = self.gen_generic(module, role, dry_run)

        dest.write(output)
        dest.close()
        logger.info(f"Wrote {len(output)} chars.")


    def gen_generic(self, module, role, dry_run=False):
        mod_name = settings.ARROYO_EXTRACTORS.get(module, None)
        if mod_name is None:
            raise CommandError(f"Module {module} not found.")
        mod = import_string(mod_name)

        objs_in_role = mod.collect_for_babel(role=role)
        logger.info(f"Tangling {len(objs_in_role)} entries for {role} role now.")

        if dry_run:
            return '\n'.join([
                    gen_obj.org_file.path
                    for gen_obj in objs_in_role
                ])

        return '\n'.join([
            gen_obj.to_babel()
            for gen_obj in objs_in_role
        ])
# Generating code using the Arroyo Generators:2 ends here
