# [[file:../../../interfaces.org::*Ingest files in bulk using the command line][Ingest files in bulk using the command line:1]]
import os.path
import glob
from itertools import filterfalse
import pdb
import graphlib
import pathlib

from generators.models import DependencyRelationship

from django.core.management.base import BaseCommand, CommandError
from django.utils.module_loading import import_string

import arroyo.arroyo_rs as native

import logging
logger = logging.getLogger(__name__)

# from roam.models import File, Keyword, Heading, Tag, Reference
from roam.models import File
from roam.core import persist_one_file, arroyo_persist_one_file, parse_doc
# Ingest files in bulk using the command line:1 ends here

# [[file:../../../interfaces.org::*Ingest files in bulk using the command line][Ingest files in bulk using the command line:2]]
from roam.core import should_file_persist
# Ingest files in bulk using the command line:2 ends here

# [[file:../../../interfaces.org::*Ingest files in bulk using the command line][Ingest files in bulk using the command line:3]]
class Command(BaseCommand):
    help = 'Ingest files in to the Roam database'
    output_transaction = True
    requires_migrations_checks = True
# Ingest files in bulk using the command line:3 ends here

# [[file:../../../interfaces.org::*Ingest files in bulk using the command line][Ingest files in bulk using the command line:4]]
    def add_arguments(self, parser):
        parser.add_argument('source_dir', type=str, default="~/org", nargs='?')

    def handle(self, *args, **kwargs):
        src = os.path.expanduser(kwargs['source_dir'])
# Ingest files in bulk using the command line:4 ends here

# [[file:../../../interfaces.org::*Ingest files in bulk using the command line][Ingest files in bulk using the command line:5]]
        files_glob = "**/*.org"
        files = glob.glob(files_glob, root_dir=src, recursive=True)
        files = map(lambda it: pathlib.Path(src).joinpath(it).expanduser(), files)
        files = filter(lambda it: it.is_file(), files)
        files = filter(lambda it: not it.is_symlink(), files)
        files = map(lambda it: str(it), files)
        files = list(files)

        logger.info(f"Collected {len(list(files))} files for parsing.")
# Ingest files in bulk using the command line:5 ends here

# [[file:../../../interfaces.org::*Ingest files in bulk using the command line][Ingest files in bulk using the command line:6]]
        docs = []
        persisted_paths = []
        sorter = graphlib.TopologicalSorter()
        for path in files:
            is_existing, need_update, doc = should_file_persist(path)
            if is_existing == True and need_update == True:
              existing = File.objects.get(path=path)
              existing.delete()
              assert doc is not None
            elif is_existing == False and need_update == True:
              assert doc is not None
            elif is_existing == True and need_update == False:
              logger.debug(f"Skipping already-parsed file at {path}")
              continue
            elif is_existing == False and need_update == False:
              logger.debug(f"Skipping invalid doc at {path}.\n")
              continue
# Ingest files in bulk using the command line:6 ends here

# [[file:../../../interfaces.org::*Ingest files in bulk using the command line][Ingest files in bulk using the command line:7]]
            DependencyRelationship.extract_ordering_from_arroyo(sorter, doc)
            the_file = persist_one_file(doc)
            docs.append(the_file)
            persisted_paths.append(path)
# Ingest files in bulk using the command line:7 ends here

# [[file:../../../interfaces.org::*Ingest files in bulk using the command line][Ingest files in bulk using the command line:8]]
        for path in sorter.static_order():
            logger.debug(f"Attempting to parse {path}.")
            doc = parse_doc(path)
            arroyo_persist_one_file(doc)
        logger.info(f"Scanned {len(files)} and parsed/persisted {len(docs)} docs.")
# Ingest files in bulk using the command line:8 ends here
