// [[file:../../../../sitemap.org::*JavaScript to set up and render the SigmaJS graph][JavaScript to set up and render the SigmaJS graph:1]]
const container = document.getElementById("sitemap-container");
fetch('/sitemap.json')
    .then(response => response.json())
    .then((data) => {
        const graph = new graphology.Graph.from(data);

        var FA2Layout = graphologyLibrary.FA2Layout;
        var forceAtlas2 = graphologyLibrary.layoutForceAtlas2;
        const sensibleSettings = forceAtlas2.inferSettings(graph);
        const layout = new FA2Layout(graph, {
            settings: sensibleSettings
        });

        const renderer = new Sigma(graph, container);

        state = {};
        function setHoveredNode(node) {
            if (node) {
                state.hoveredNode = node;
                state.hoveredNeighbors = new Set(graph.neighbors(node));
            } else {
                    state.hoveredNode = undefined;
                    state.hoveredNeighbors = undefined;
                }
        
                // Refresh rendering:
                renderer.refresh();
            }
        
            renderer.on("enterNode", ({ node }) => {
                setHoveredNode(node);
            });
            renderer.on("leaveNode", () => {
                setHoveredNode(undefined);
            });
        
            // Render nodes accordingly to the internal state:
            // 1. If a node is selected, it is highlighted
            // 2. If there is query, all non-matching nodes are greyed
            // 3. If there is a hovered node, all non-neighbor nodes are greyed
            renderer.setSetting("nodeReducer", (node, data) => {
                const res = { ...data };
        
                if (state.hoveredNeighbors && !state.hoveredNeighbors.has(node) && state.hoveredNode !== node) {
                    res.label = "";
                    res.color = "#f6f6f6";
                }
        
                if (state.selectedNode === node) {
                    res.highlighted = true;
                } else if (state.suggestions && !state.suggestions.has(node)) {
                    res.label = "";
                    res.color = "#f6f6f6";
                }
        
                return res;
            });
        
            // Render edges accordingly to the internal state:
            // 1. If a node is hovered, the edge is hidden if it is not connected to the
            //    node
            // 2. If there is a query, the edge is only visible if it connects two
            //    suggestions
            renderer.setSetting("edgeReducer", (edge, data) => {
                const res = { ...data };
        
                if (state.hoveredNode && !graph.hasExtremity(edge, state.hoveredNode)) {
                    res.hidden = true;
                }
        
                if (state.suggestions && (!state.suggestions.has(graph.source(edge)) || !state.suggestions.has(graph.target(edge)))) {
                    res.hidden = true;
                }
        
                return res;
            });
        

        layout.start();
        setTimeout(() => {
            layout.stop()
        }, 5000);

        renderer.on("clickNode", ({ node }) => {
            var realnode = graph._nodes.get(node);
            window.location = realnode.attributes.href;
        });

        return {renderer, graph}
    });
// JavaScript to set up and render the SigmaJS graph:1 ends here
